// Hatch selected closed curves
// Version from March 4, 2015

// NOTE
// Works in TOP view only
// If you want to hatch several closed curves at once they must be all on the same Z plane.

// HINT
// You can hide all hatch lines In the Scene browser under Objects.
// Or you can select and delete or color all hatch lines.

// Modified 2/27/2021 - Michael Gibson - added rotation option.

function rotatept( origin, pt, angle_degrees )
{
	if ( angle_degrees == 0.0 )
		return pt;

	var x = pt.x - origin.x;
	var y = pt.y - origin.y;

	var angle_radians = angle_degrees * Math.PI / 180.0;
	var cosangle = Math.cos( angle_radians );
	var sinangle = Math.sin( angle_radians );

	var newx = (x * cosangle) - (y * sinangle);
	var newy = (x * sinangle) + (y * cosangle);

	newx += origin.x;
	newy += origin.y;

	return moi.vectorMath.createPoint( newx, newy, pt.z );
}

function hatchit()
{
	var hatchlineOffset = moi.ui.commandUI['offset'].numericValue * 1.0;
	
	var objects = moi.geometryDatabase.getSelectedObjects();

	var line_factory = moi.command.createFactory( "line" );
	
	//Prepare hatch over object
	var arraydir_factory = moi.command.createFactory( "arraydir" );
	arraydir_factory.setInput( 5, 'Offset, Extent' ); //Mode
	
	//Prepare cross hatch over object
	var arraydir2_factory = moi.command.createFactory( "arraydir" );
	arraydir2_factory.setInput( 5, 'Offset, Extent' ); //Mode
	
	//Prepare trim hatch lines
	var trim_factory = moi.command.createFactory( 'booleanintersection' );
	trim_factory.setInput( 1, objects );	

	var rotation_origin = objects.getBoundingBox().center;


	moi.geometryDatabase.deselectAll();
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		
		if ( moi.ui.commandUI.offset.value != 0 )
		{
			var rotation = moi.ui.commandUI.rotation.value;
			var bbox;

			if ( rotation == 0.0 )
			{
				bbox = objects.getHighAccuracyBoundingBox();
			}
			else
			{
				// Rotate objects to get bounding box.
				var rotate_factory = moi.command.createFactory( 'rotate' );
				var frame = moi.vectorMath.createTopFrame();
				frame.origin = rotation_origin;
				rotate_factory.setInput( 0, objects );
				rotate_factory.setInput( 1, frame );
				rotate_factory.setInput( 2, -rotation );

				var rotated_objs = rotate_factory.calculate();
				bbox = rotated_objs.getHighAccuracyBoundingBox();
				rotate_factory.cancel();
			}
						
			var height = bbox.max.y - bbox.min.y


			//Calculate first hatch line
			var startpt = moi.vectorMath.createPoint( bbox.min.x, bbox.max.y, bbox.min.z );
			var endpt = moi.vectorMath.createPoint( bbox.min.x - height, bbox.min.y, bbox.min.z )
			startpt = rotatept( rotation_origin, startpt, rotation );
			endpt = rotatept( rotation_origin, endpt, rotation );
	
			line_factory.setInput( 0, startpt );
			line_factory.setInput( 1, endpt );
			var line1 = line_factory.calculate();


			//Calculate first cross hatch line
			startpt = moi.vectorMath.createPoint( bbox.min.x - height, bbox.max.y, bbox.min.z );
			endpt = moi.vectorMath.createPoint( bbox.min.x, bbox.min.y, bbox.min.z );
			startpt = rotatept( rotation_origin, startpt, rotation );
			endpt = rotatept( rotation_origin, endpt, rotation );

			line_factory.setInput( 0, startpt );
			line_factory.setInput( 1, endpt );
			var line2 = line_factory.calculate();


			var basept = moi.vectorMath.createPoint( bbox.min.x - height, bbox.min.y, bbox.min.z );
			var extentpt = moi.vectorMath.createPoint( bbox.max.x, bbox.min.y, bbox.min.z );
			basept = rotatept( rotation_origin, basept, rotation );
			extentpt = rotatept( rotation_origin, extentpt, rotation );

			arraydir_factory.setInput( 0, line1 );
			arraydir_factory.setInput( 2, basept ); //BasePt
			arraydir_factory.setInput( 4, extentpt ); //ExtentPt


			basept = moi.vectorMath.createPoint( bbox.min.x, bbox.min.y, bbox.min.z );
			extentpt = moi.vectorMath.createPoint( bbox.max.x + height, bbox.min.y, bbox.min.z );
			basept = rotatept( rotation_origin, basept, rotation );
			extentpt = rotatept( rotation_origin, extentpt, rotation );

			arraydir2_factory.setInput( 0, line2 );
			arraydir2_factory.setInput( 2, basept ); //BasePt
			arraydir2_factory.setInput( 4, extentpt ); //ExtentPt


			hatchlineOffset = moi.ui.commandUI['offset'].numericValue * 1.0;
			
			var left_right = moi.ui.commandUI['left_right'].value;
			var right_left = moi.ui.commandUI['right_left'].value;

			if ( left_right )
			{
				//Hatch over object

				var offsetpt = moi.vectorMath.createPoint( bbox.min.x - height + hatchlineOffset, bbox.min.y, bbox.min.z )
				offsetpt = rotatept( rotation_origin, offsetpt, rotation );

				arraydir_factory.setInput( 3, offsetpt ); //OffsetPt
				arraydir_factory.update();
				arraydir_factory.getCreatedObjects().setProperty( 'selected', true );
			} else {

				var offsetpt = moi.vectorMath.createPoint( bbox.min.x - height + 0, bbox.min.y, bbox.min.z )
				offsetpt = rotatept( rotation_origin, offsetpt, rotation );

				arraydir_factory.setInput( 3, offsetpt ); //OffsetPt
				arraydir_factory.update();	
				arraydir_factory.getCreatedObjects().setProperty( 'selected', false );
			}
			
			if ( right_left )
			{	
				//Cross hatch over object
				var offsetpt = moi.vectorMath.createPoint( bbox.min.x + hatchlineOffset, bbox.min.y, bbox.min.z )
				offsetpt = rotatept( rotation_origin, offsetpt, rotation );

				arraydir2_factory.setInput( 3, offsetpt ); //OffsetPt
				arraydir2_factory.update();
				arraydir2_factory.getCreatedObjects().setProperty( 'selected', true );
			} else {

				var offsetpt = moi.vectorMath.createPoint( bbox.min.x + 0, bbox.min.y, bbox.min.z )
				offsetpt = rotatept( rotation_origin, offsetpt, rotation );

				arraydir2_factory.setInput( 3, offsetpt ); //OffsetPt
				arraydir2_factory.update();	
				arraydir2_factory.getCreatedObjects().setProperty( 'selected', false );
			}
			
				//Trim hatch lines
				trim_factory.setInput( 0, moi.geometryDatabase.getSelectedObjects() );
				trim_factory.update();

		}
	
		if ( !dlg.waitForEvent() )
			return false; // Canceled
			
		if ( dlg.event == 'done' )
			break;
	
	}
	
	hatchlines = trim_factory.getCreatedObjects();
	hatchlines.setProperty( 'name', "Hatch lines" );	
	trim_factory.commit();
	

}

hatchit();

